declare type CompareFunction<T> = (a: T, b: T) => number;
/**
 * 比较器
 */
declare class Comparator<T> {
    private compare;
    /**
     * Constructor.
     *
     * @param compareFunction It may be custom compare function that, let's say may compare custom objects together.
     */
    constructor(compareFunction?: CompareFunction<T>);
    /**
     * Default comparison function. It just assumes that "a" and "b" are strings or numbers.
     * @param a
     * @param b
     * @returns
     */
    static defaultCompareFunction(a: string | number, b: string | number): 0 | 1 | -1;
    /**
     * Checks if two variables are equal.
     *
     * @param a
     * @param b
     * @return
     */
    equal(a: T, b: T): boolean;
    /**
     * Checks if variable "a" is less than "b".
     *
     * @param a
     * @param  b
     * @return
     */
    lessThan(a: T, b: T): boolean;
    /**
     * Checks if variable "a" is greater than "b".
     *
     * @param a
     * @param  b
     * @return
     */
    greaterThan(a: T, b: T): boolean;
    /**
     * Checks if variable "a" is less than or equal to "b".
     *
     * @param a
     * @param  b
     * @return
     */
    lessThanOrEqual(a: T, b: T): boolean;
    /**
     * Checks if variable "a" is greater than or equal to "b".
     *
     * @param a
     * @param b
     * @return
     */
    greaterThanOrEqual(a: T, b: T): boolean;
    /**
     * Reverses the comparison order.
     */
    reverse(): void;
}

export { Comparator };
export as namespace feng3d;
