/*!
 * @feng3d/data-structures-and-algorithms - v0.0.1
 * Compiled Thu, 02 Jun 2022 13:57:51 UTC
 *
 * @feng3d/data-structures-and-algorithms is licensed under the MIT License.
 * http://www.opensource.org/licenses/mit-license
 */
this.feng3d = this.feng3d || {};
var _feng3d_data_structures_and_algorithms = (function (exports) {
  'use strict';

  /**
   * 比较器
   */
  var Comparator = /** @class */ (function () {
      /**
       * Constructor.
       *
       * @param compareFunction It may be custom compare function that, let's say may compare custom objects together.
       */
      function Comparator(compareFunction) {
          this.compare = compareFunction || Comparator.defaultCompareFunction;
      }
      /**
       * Default comparison function. It just assumes that "a" and "b" are strings or numbers.
       * @param a
       * @param b
       * @returns
       */
      Comparator.defaultCompareFunction = function (a, b) {
          if (a === b) {
              return 0;
          }
          return a < b ? -1 : 1;
      };
      /**
       * Checks if two variables are equal.
       *
       * @param a
       * @param b
       * @return
       */
      Comparator.prototype.equal = function (a, b) {
          return this.compare(a, b) === 0;
      };
      /**
       * Checks if variable "a" is less than "b".
       *
       * @param a
       * @param  b
       * @return
       */
      Comparator.prototype.lessThan = function (a, b) {
          return this.compare(a, b) < 0;
      };
      /**
       * Checks if variable "a" is greater than "b".
       *
       * @param a
       * @param  b
       * @return
       */
      Comparator.prototype.greaterThan = function (a, b) {
          return this.compare(a, b) > 0;
      };
      /**
       * Checks if variable "a" is less than or equal to "b".
       *
       * @param a
       * @param  b
       * @return
       */
      Comparator.prototype.lessThanOrEqual = function (a, b) {
          return this.lessThan(a, b) || this.equal(a, b);
      };
      /**
       * Checks if variable "a" is greater than or equal to "b".
       *
       * @param a
       * @param b
       * @return
       */
      Comparator.prototype.greaterThanOrEqual = function (a, b) {
          return this.greaterThan(a, b) || this.equal(a, b);
      };
      /**
       * Reverses the comparison order.
       */
      Comparator.prototype.reverse = function () {
          var compareOriginal = this.compare;
          this.compare = function (a, b) { return compareOriginal(b, a); };
      };
      return Comparator;
  }());

  exports.Comparator = Comparator;

  Object.defineProperty(exports, '__esModule', { value: true });

  return exports;

})({});
Object.assign(this.feng3d, _feng3d_data_structures_and_algorithms);
//# sourceMappingURL=index.js.map
